#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# breakpoint display window for the SmartGDB GUI 
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# Author: Charles Dymond  <cdy@ittc.ukans.edu>
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

global lastBline
set lastBline 0

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# Clears all breakoints
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc clear_all { } {
    set breaks [get_breaks]
    set breaks [split $breaks \n]
    foreach i $breaks {
	set break_num ""
	set char [string index $i 0]
	set pos 1
	while { ($char >= "0")&&($char <= "9") } {
	    append break_num $char
	    set char [string index $i $pos]
	    incr pos
	}
       [ d $break_num ]
    }
}


proc select { } {

    set name .breaks.txt
    set line_num [get_pos $name]
    highlight_line $name $line_num

}

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# deletes selected breakpoint
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc delete_breakpoint { } {

    global lastLineSelected
    .breaks.txt mark set first $lastLineSelected.0 
    .breaks.txt mark set last $lastLineSelected.end

    set line [.breaks.txt get first last]
    set break_num ""
    set char [string index $line 0]
    set pos 1
    while { ($char >= "0")&&($char <= "9") } {
	append break_num $char
	set char [string index $line $pos]
	incr pos
    }
    
    [ d $break_num ]
   
}

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# returns a list of all curently set breakpoints
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc get_breaks { } {
    set blurb "[ i break ]"
    set blurb [ split $blurb \n ]
    set output ""
    
    foreach x $blurb {
	set char [string index $x 0]
	set pos 1
	if { ($char >= "0")&&($char <= "9") } {
	    while { ($char >= "0")&&($char <= "9") } {
		append output $char
		set char [string index $x $pos]
		incr pos
	    }
	    append output "  "
	}
	set pos [string length $x]
	set pos [expr $pos-1]
	set char [string index $x $pos]
	if { ($char >= "0")&&($char <= "9") } {
	    set filename ""
	    while { $char != "/" } {
		set filename "$char$filename"
		set pos [expr $pos-1]
		set char [string index $x $pos]
	    }
	append output $filename
	append output \n
	}
    }

    return $output
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# gets a list of all currently set breakpoints and
# displays them in a list box.  A breakpoint can be 
# selected and deleted from this list
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc show_breakpoints { } {
 
    global breakWindow
    set breakWindow 1

    global lastBline
    set lastBline 0

    global commandWindow

    toplevel .breaks
    wm title .breaks "Breakpoints"
    wm geometry .breaks 250x300+15+15

    text .breaks.txt \
	-xscrollcommand ".breaks.xscroll set" \
	-yscrollcommand ".breaks.yscroll set" \
	-fg red -bg white -wrap none
    scrollbar .breaks.xscroll -orient horizontal \
	-command ".breaks.txt xview"
    scrollbar .breaks.yscroll -orient vertical \
	-command ".breaks.txt yview"

    frame .breaks.bframe
    button .breaks.bframe.b1 -text "Dismiss" -command { set breakWindow 0; destroy .breaks }
    button .breaks.bframe.b2 -text "Delete" -command { 
	delete_breakpoint
	insert_text .breaks.txt [get_breaks] 
    }    

    pack .breaks.bframe.b1 .breaks.bframe.b2 -side left

    grid columnconfigure .breaks 0 -weight 1
    grid columnconfigure .breaks 1 -weight 0; 
    grid rowconfigure .breaks 0 -weight 1
    grid rowconfigure .breaks 1 -weight 0; 
    grid rowconfigure .breaks 2 -weight 0

    grid .breaks.txt -column 0 -row 0 -sticky nsew
    grid .breaks.yscroll -column 1 -row 0 -sticky ns
    grid .breaks.xscroll -column 0 -row 1 -sticky ew
    grid .breaks.bframe -column 0 -row 2 -columnspan 2 -sticky nsew

    set breaks [get_breaks]
    insert_text .breaks.txt $breaks

    bind .breaks.txt <Button-1> { select }

    bind .breaks.txt <Double-Button-1> {
	set line [get_selected_line .breaks.txt [get_pos .breaks.txt]]
	set line_num [lindex [split $line :] 1]
	set line [lindex [split $line " "] 2]
	set fileName [lindex [split $line :] 0]
	display_file $fileName
        .smartgdb.text_frame.text see  $line_num.0
	highlight_break $line_num
    }
}


