#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# main window for the SmartGDB GUI
# Author: Charles Dymond  <cdy@ittc.ukans.edu>
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-


global GotSelection
set GotSelection 0

global selectedLineNumber
set selectedLineNumber 1

global lastLineSelected
set lastLineSelected 0

global lastWindowSelected
set lastWindowSelected ""

global currentSourceFile
set currentSourceFile ""

global NumOfLines
set NumOfLines 0

global LastBreakLine
set LastBreakLine 0

global sourceWindow
set sourceWindow 0

global breakWindow
set breakWindow 0

global threadWindow
set threadWindow 0

global lastMline
set lastMline 0

global runningThread
set runningThread 0

global oneThread
set oneThread 0

global threadID
set threadID 0

global W-Package
set WPackage ""

global lastSline
set lastSline 0

global contlist
set contlist "" 

global PATH
set PATH ${tcl_library}

source $PATH/interface/source.tcl
source $PATH/interface/break.tcl
source $PATH/interface/user.tcl
source $PATH/interface/command.tcl
source $PATH/interface/parse_html.tcl

toplevel .smartgdb

#
# Menu Widgets
#
frame .smartgdb.menu_frame -relief raised

# File Menu
menubutton .smartgdb.menu_frame.file -text "File" -underline 0 \
    -menu .smartgdb.menu_frame.file.menu -justify left
menu .smartgdb.menu_frame.file.menu
.smartgdb.menu_frame.file.menu add command -label "Scripts" -command { setup_scripts }
.smartgdb.menu_frame.file.menu add command -label "Run" -command { [ run ] }
.smartgdb.menu_frame.file.menu add command -label "Exit Window" -command { quit_debugger }
.smartgdb.menu_frame.file.menu add command -label "Exit Smartgdb" -command "exit"



# Options Menu
menubutton .smartgdb.menu_frame.options -text "Options" -underline 0 \
    -menu .smartgdb.menu_frame.options.menu
menu .smartgdb.menu_frame.options.menu
.smartgdb.menu_frame.options.menu add command -label "Command Window" -command { gdb_window }
.smartgdb.menu_frame.options.menu add command -label "Run Setup Script" -command { 
    insert_text .smartgdb.status "Loading..."
    update
    [source ~/setup.tcl] 
    insert_text .smartgdb.status "Done"
}
.smartgdb.menu_frame.options.menu add command -label "Toggle Left Toolbar"  -command toggle_w_frame
.smartgdb.menu_frame.options.menu add command -label "Show Source Files" -command goto_source
.smartgdb.menu_frame.options.menu add command -label "Hide Logo" -command {wm withdraw .}

# Breakpoint Menu
menubutton .smartgdb.menu_frame.breakpoint -text "Breakpoints" -underline 0 \
    -menu .smartgdb.menu_frame.breakpoint.menu
menu .smartgdb.menu_frame.breakpoint.menu
set breakmenu .smartgdb.menu_frame.breakpoint.menu 
$breakmenu add command -label "Set Breakpoint" -command set_bp
$breakmenu add command -label "Clear Current Breakpoint" -command clear_breakpoint
$breakmenu add command -label "Clear All Breakpoints" -command clear_all
$breakmenu add command -label "Show Breakpoints" -command show_breakpoints


# Help Menu
menubutton .smartgdb.menu_frame.help -text "Help" -underline 0 \
    -menu .smartgdb.menu_frame.help.menu -justify left
menu .smartgdb.menu_frame.help.menu
.smartgdb.menu_frame.help.menu add command -label "Help"  -command {
    ############################################
    #####+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-#####
    ##### location of smartgdb help files  #####
    #####+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-#####
    ############################################
    display_html "$PATH/interface/html/links.html"
}



#
#  Text Window Widgets
#
frame .smartgdb.text_frame
text .smartgdb.text_frame.text \
    -xscrollcommand ".smartgdb.text_frame.xscroll set" \
    -yscrollcommand ".smartgdb.text_frame.yscroll set" \
    -fg $normalFG  -bg $normalBG -wrap none -state disabled 
scrollbar .smartgdb.text_frame.xscroll -orient horizontal -command \
    ".smartgdb.text_frame.text xview"
scrollbar .smartgdb.text_frame.yscroll -orient vertical -command \
    ".smartgdb.text_frame.text yview"

#
# Southern button Widgets
#
frame .smartgdb.s_button_frame
button .smartgdb.s_button_frame.quit -text "Quit" -command { quit_debugger }
button .smartgdb.s_button_frame.continue -text "Continue" -command  { 
    do_cont
}
button .smartgdb.s_button_frame.step -text "Step" -command do_step
button .smartgdb.s_button_frame.jump -text "Jump" -command do_jump
button .smartgdb.s_button_frame.next -text "Next" -command do_next
button .smartgdb.s_button_frame.add_break -text "Set Breakpoint" -command set_bp

.smartgdb.s_button_frame.quit configure -fg red
.smartgdb.s_button_frame.continue configure -fg red
.smartgdb.s_button_frame.step configure -fg red
.smartgdb.s_button_frame.jump configure -fg red
.smartgdb.s_button_frame.next configure -fg red
.smartgdb.s_button_frame.add_break configure -fg red

#
# Western button Widgets
#
frame .smartgdb.w_frame -relief groove


#
# Pack and grid the widgets
#
pack .smartgdb.menu_frame.file .smartgdb.menu_frame.options .smartgdb.menu_frame.breakpoint  -side left
pack .smartgdb.menu_frame.help -side right

grid columnconfigure .smartgdb.text_frame 0 -weight 1
grid columnconfigure .smartgdb.text_frame 1 -weight 0; 
grid rowconfigure .smartgdb.text_frame 0 -weight 1
grid rowconfigure .smartgdb.text_frame 1 -weight 0; 

grid .smartgdb.text_frame.text -column 0 -row 0 -sticky nsew
grid .smartgdb.text_frame.yscroll -column 1 -row 0 -sticky nsew
grid .smartgdb.text_frame.xscroll -column 0 -row 1 -sticky ewns

pack .smartgdb.s_button_frame.quit \
    .smartgdb.s_button_frame.continue \
    .smartgdb.s_button_frame.step\
    .smartgdb.s_button_frame.jump \
    .smartgdb.s_button_frame.next\
    .smartgdb.s_button_frame.add_break \
    -side left -fill x -expand y

text .smartgdb.status \
    -fg red  -wrap none -state disabled -height 1 -width 80


pack .smartgdb.menu_frame -side top -fill x
pack .smartgdb.status -side top -anchor w
pack .smartgdb.w_frame -side left
pack .smartgdb.s_button_frame -side bottom
pack .smartgdb.text_frame -side right -expand y -fill both

insert_text .smartgdb.status "Ready (no program specified)"

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# uncomment this to see error message
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

#   rename unknown unknown-tcl

#   proc unknown { args } {
#     puts "unknown: $args"
#   }


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# close all open windows then exit
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc quit_debugger { } {
    global sourceWindow
    global breakWindow
    global commandWindow

    if [ winfo exists .sources ] {destroy .sources} 
    if [ winfo exists .breaks ] {destroy .breaks}
    if [ winfo exists .gdb ] {destroy .gdb}

    foreach i [winfo children .smartgdb] {
	destroy $i
    }

    destroy .smartgdb
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# shows/hides a selected package 
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc toggle_w_frame {} {

    global WPackage

    if [winfo ismapped .smartgdb.w_frame] {
	pack forget .smartgdb.w_frame
    } else {
	if {$WPackage != ""} {
	    eval $WPackage 
	}
    }
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# deletes any existing text and replaces it with a 
# listing of the current source file
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

proc get_text { } {

    global lastLineSelected
    global NumOfLines
    global lastMline

    global currentSourceFile
    
    get_current_source
    set foo "[ l 1,10000 ]"
    set name .smartgdb.text_frame.text
    insert_text $name $foo
    set lastLineSelected 0
    set lastMline 0
    set target .smartgdb.text_frame.text
    scan [ $target index end ] %d numlines
    set NumOfLines $numlines
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# extracts the name of the current source file from
# the output of the smartgdb "i source" command
# and updates the name of the tcl window with the name
# of the source file.
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc get_current_source { } {

    global currentSourceFile

    set filename ""
    set blurb "[ i source ]"
    set tmp1 [ lindex [ split $blurb \n ] 0 ]
    set filename [lindex [split $blurb /] 2]
    set filename [lindex [split $filename \n] 0]
    set currentSourceFile $filename
    wm title .smartgdb "$currentSourceFile"
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# this procedure highlights a breakpoint
# line in red
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc highlight_break { line_num } {

    global LastBreakLine
    global breakBG
    global breakFG

    set LastBreakLine $line_num

    if { $line_num != 0 } {
	.smartgdb.text_frame.text mark set first $line_num.0
	.smartgdb.text_frame.text mark set last $line_num.end
	.smartgdb.text_frame.text tag remove ls first last
	.smartgdb.text_frame.text tag add bp first last
	.smartgdb.text_frame.text tag configure bp -background $breakBG \
	    -foreground $breakFG -relief raised -borderwidth 1
    }
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# extracts the source file name and line number of the
# last breakpoint reached.  The file is displayed in
# the main window with the breakline highlighted
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc goto_break { blurb } {

    set tmp1 [ lindex [ split $blurb \n ] 3 ]

    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    #get some text to display
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    get_text

    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # find line number of break point
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    set first_ch [string index $tmp1 0]
    set line_num ""

    if { $first_ch != " " } {
	set char $first_ch
	set pos 1

	while { ($char >="0")&&($char <= "9")  } {
	    append line_num  $char 
	    set char [string index $tmp1 $pos]
	    incr pos
	}
    } else {
	set pos [string length $tmp1]
	set pos [expr $pos-1]
	set char [string index $tmp1 $pos]

	while { $char != ":" } {
	    set line_num "$char$line_num"
	    set pos [expr $pos-1]
	    set char [string index $tmp1 $pos]
	}	
	
    }
    
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # line_num is now the line number of
    # the current break point
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    return $line_num
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# clears the current breakpoint and un highlights the line
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc clear_breakpoint { } {

    global LastBreakLine
    global commandWindow

    set blurb "[ clear ]"
    
    if { $commandWindow==1 } {
	echo_output $blurb
    }    

    set line_num $LastBreakLine
    if { $line_num != 0 } {
	.smartgdb.text_frame.text mark set lfirst $line_num.0
	.smartgdb.text_frame.text mark set llast $line_num.end
	.smartgdb.text_frame.text tag remove bp lfirst llast
    }
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# continue to next break point and get breakpoint
# info to find correct line number in source
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc do_cont { } {

    global commandWindow
    global threadWindow

    insert_text .smartgdb.status "Running... "
    update
    set blurb "[ c ]"
    set line_num [ goto_break $blurb ]

    if { $commandWindow==1 } {
	echo_output $blurb
    }

    .smartgdb.text_frame.text see $line_num.0
    highlight_break $line_num

    set blurb [split $blurb \n]
    set blurb [lindex $blurb 2]
    set bp [lindex [split $blurb ","] 0]
    insert_text .smartgdb.status "at $bp"
    exec_cont

}
 

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# executes each of a list of commands or procedures
# to be executed when a the continue,next, step or jump 
# commands are issued this allows packages to append their 
# own set of commands to the list of things to do when a 
# breakpoint is reached
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc exec_cont { } {
    global contlist
    foreach i $contlist {
	if { $i != "" } {
	    [$i]
	}
    }
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# Procedure: set_bp
# Purpose:  this procedure will set a 
# breakpoint at the currently highlighted line
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
proc set_bp { } {

    global selectedLineNumber
    global commandWindow

    set blurb "[ b $selectedLineNumber ]"
    if { $commandWindow == 1 } {
	echo_output $blurb
    }
    set tmp $selectedLineNumber
    highlight_break $tmp
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# Procedure: do_step
# Purpose: invokes the smartgdb "step"
# command when button is pushed
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
proc do_step { } {

    global commandWindow
    set widgetName .smartgdb.text_frame.text

    set blurb "[ s ]"

    if { $commandWindow==1 } {
	echo_output $blurb
    }

    get_text
    set line_num [get_line_number $blurb]
    $widgetName see $line_num.0
    highlight_line $widgetName $line_num

    exec_cont

}   


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# Procedure: do_next
# Purpose: invokes the smartgdb "next"
# command when button is pushed
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
proc do_next { } {

    global commandWindow

    set widgetName .smartgdb.text_frame.text
    set blurb "[ n ]"

    if { $commandWindow==1 } {
	echo_output $blurb
    }

    get_text
    set line_num [get_line_number $blurb]
    $widgetName see $line_num.0
    highlight_line $widgetName $line_num

    exec_cont

}

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# executes jump command in smartgdb
# highlights line jumped to and re positions screen
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc jumprt { jumpto } {

    set blurb "[ jump $jumpto ]"
    destroy .jumpto

    .smartgdb.text_frame.text see $jumpto.0
    highlight_line .smartgdb.text_frame.text $jumpto

    exec_cont
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# creates entry box for line number to jump to
# calls "jumprt" to execute smartgdb command
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc do_jump { } {

    set jumpto 1

    toplevel .jumpto
    wm title .jumpto "Jump"
    
    frame .jumpto.jmpfr -relief raised

    label .jumpto.jmpfr.label -text "Jump to:"
    entry .jumpto.jmpfr.entry -width 20 -relief sunken -bd 2 \
	-textvariable jumpto -bg white -fg blue
    button .jumpto.jmpfr.b -text "ok" -command {jumprt $jumpto}
    button .jumpto.jmpfr.b2 -text "cancel" -command { destroy .jumpto }
    pack .jumpto.jmpfr.label .jumpto.jmpfr.entry .jumpto.jmpfr.b  .jumpto.jmpfr.b2 \
	-side left -padx 1m -pady 2m 
    pack .jumpto.jmpfr
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# displays a directory browser for choosing which 
# tcl scripts to run
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc setup_scripts { } {

    global normalFG
    global normalBG
    global lastSline
    
    toplevel .scripts
    wm title .scripts "Scripts"
    wm geometry .scripts 200x370+15+15

    text .scripts.txt \
	-fg $normalFG -bg $normalBG -wrap none \
	-xscrollcommand ".scripts.xscroll set" \
	-yscrollcommand ".scripts.yscroll set"

    .scripts.txt configure -state disabled

    scrollbar .scripts.xscroll \
	-orient horizontal \
	-command ".scripts.txt xview"

    scrollbar .scripts.yscroll \
	-orient vertical \
	-command ".scripts.txt yview"

    frame .scripts.bframe

    button .scripts.bframe.dismiss \
	-text "Dismiss" \
	-command {destroy .scripts}

    pack .scripts.bframe.dismiss \
	-side left -fill y -expand n
    
    grid columnconfigure .scripts 0 -weight 1
    grid columnconfigure .scripts 1 -weight 0    

    grid rowconfigure .scripts 0 -weight 1
    grid rowconfigure .scripts 1 -weight 0
    grid rowconfigure .scripts 2 -weight 0

    grid .scripts.txt -column 0 -row 0 -sticky nsew
    grid .scripts.xscroll -column 0 -row 1 -sticky ew
    grid .scripts.yscroll -column 1 -row 0 -sticky ns
    grid .scripts.bframe -column 0 -row 2 -sticky nsew

    cd ~/
    insert_dir

    bind .scripts.txt <Button-1> {
	set name .scripts.txt
	set line [get_pos $name]
	highlight_line $name $line
    }

    bind .scripts.txt <Double-Button-1> {
	set name .scripts.txt 
	set line [get_pos $name]
	if { $line == 2 } {
	    cd ..
	    insert_dir
	} else { 
	    set line [get_selected_line .scripts.txt [get_pos .scripts.txt]]
	    regsub -all "<" $line "" line
	    regsub -all ">" $line "" line
	    if { [ file isdirectory $line] != 0 } {
		cd $line 
		insert_dir
	    } else { 
		insert_text .smartgdb.status "Loading..."
		update
		[source $line]
		insert_text .smartgdb.status "Done"
		destroy .scripts
		break
	    }
	}   
	break
    }
}

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# inserts a listing of the current directory, directory
# names are enclosed in angle brackets and tcl scripts
# (those ending in .tcl anyway) are shown in red and
# are underlined
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc insert_dir { } {
    .scripts.txt configure -state normal
    .scripts.txt delete 1.0 end
    .scripts.txt insert insert "."
    .scripts.txt insert insert \n
    .scripts.txt insert insert ".."
    .scripts.txt insert insert \n

    set line 3
    
    set dirlist [glob -nocomplain *]
    set dirlist [split $dirlist " "] 
    
    foreach i $dirlist {
	if { [file isdirectory $i] != 0 } {
	    set i <$i>
	    .scripts.txt insert insert $i
	    set name .scripts.txt
	    $name mark set lfirst $line.0
	    $name mark set llast $line.end
	    $name tag add isdir lfirst llast
	    $name tag configure isdir -foreground darkblue
	} elseif { [string match "*.tcl" $i] == 1 } {
	    set name .scripts.txt
	    .scripts.txt insert insert $i
	    $name mark set lfirst $line.0
	    $name mark set llast $line.end
	    $name tag add istcl lfirst llast
	    $name tag configure istcl -foreground red \
		-underline 1
	} else { 
	    .scripts.txt insert insert $i
	}
	.scripts.txt insert insert \n
	incr line
    }
    .scripts.txt configure -state disabled
}

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# a single click on a line will highlight it
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
bind .smartgdb.text_frame.text <Button-1> { 
    set widgetName .smartgdb.text_frame.text
    set line_num [get_pos $widgetName ]
    highlight_line $widgetName $line_num
}

bind .smartgdb.text_frame.text <B1-Motion> { break }

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# keybindings to invoke buttons
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
bind .smartgdb <KeyPress-c> {
    .smartgdb.s_button_frame.continue flash
    .smartgdb.s_button_frame.continue invoke
}

bind .smartgdb <KeyPress-q> { 
    .smartgdb.s_button_frame.quit flash
    .smartgdb.s_button_frame.quit invoke
}

bind .smartgdb <KeyPress-s> { 
    .smartgdb.s_button_frame.step flash
    .smartgdb.s_button_frame.step invoke
}

bind .smartgdb <KeyPress-j> { 
    .smartgdb.s_button_frame.jump flash
    .smartgdb.s_button_frame.jump invoke
}

bind .smartgdb <KeyPress-n> { 
    .smartgdb.s_button_frame.next flash
    .smartgdb.s_button_frame.next invoke
}

bind .smartgdb <KeyPress-k> { exit }


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# keybindings to move the highlighted line up and down
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
bind .smartgdb <Down> {
    global lastLineSelected
    set lastLineSelected [incr lastLineSelected]
    highlight_line .smartgdb.text_frame.text $lastLineSelected
}

bind .smartgdb <Up> {
    global lastLineSelected
    set lastLineSelected [expr $lastLineSelected-1]
    highlight_line .smartgdb.text_frame.text $lastLineSelected
}

bind .smartgdb.status <Button-1> {
    break
}

bind .smartgdb.status <Double-Button-1> {
    break
}

bind .smartgdb.status <B1-Motion> {
    break
}

bind .smartgdb <B1-Motion> {
    break
}


