#
#
#
#  parse_html.tcl
#
#  Purpose:      This script parses an html file and outputs it into a tk text window.  It is intended to display simple html as a 
#
#  Author:       Aaron Hoyt (hoyt@ittc.ukans.edu)
#
#  Notes:        This is a really big hack.  If you are wanting a full fledged tcl html package find it elsewhere. 
#                If we encounter an external reference we send it to netscape.
#                The parser should be implemented with a stack rather than state variables.  This would help with reliability.
#
#  Limitations:  lots...
#                All multiline tags are ignored ( this would be simple to fix, but I don't care )
#                Improperly nested or unmatcehd tags are not dealt with very well (ie not at all).
#                Internal document anchors don't work ( this would be simple to fix, and I would really like to, but it is not that important right now)
#                Slooooooooooooooowwwwwwwwwwwwwwwwwwwwwwwwwwwww on very large pages.
#                Many more.
#                
#  Entry Point:  parse_html
#                parse_html should be passed the name of the html file to be displayed
#             
#
global startup_directory
global current_directory
global current 
global mark_start
global mark_end
global title_start
global title_end
global title_text
global split_data
global tab_stop
global bold
global head1
global head2
global head3
global head4
global head5
global head6
global italic
global preformat
global code
global tag_number
global bold_start
global bold_end
global italics_start
global italics_end
global head1_start
global head1_end
global head2_start
global head2_end
global head3_start
global head3_end
global head4_start
global head4_end
global head5_start
global head5_end
global head6_start
global head6_end
global center_start
global center_end
global link_start
global link_end 
global under_start
global under_end
global over_start
global over_end
global link_value
global history
global current_history

set bold 0
set head1 0
set head2 0
set head3 0
set head4 0
set head5 0
set head6 0
set italic 0
set preformat 0
set code 0
set tag_number 0
set bold_start 0
set bold_end 0
set italics_start 0
set italics_end 0
set head1_start 0
set head1_end 0
set head2_start 0
set head2_end 0
set head3_start 0
set head3_end 0
set head4_start 0
set head4_end 0
set head5_start 0
set head5_end 0
set head6_start 0
set head6_end 0
set center_start 0
set center_end 0
set link_start 0
set link_end 0
set under_start 0
set under_end 0
set over_start 0
set over_end 0
set startup_directory [pwd]
set current_directroy $startup_directory
set history ""

#
#  decr:  
#
#  purpose:  decrement a variable
#
proc decr { value } {
    upvar $value a
    incr a -1
}

#
#  get_font:
#
#  purpose:  Examines the current state of the html page and returns the 
#            appropriate font for that state.
#
proc get_font {} {
    global bold
    global italic
    global preformat
    global code
    global head1
    global head2
    global head3
    global head4
    global head5
    global head6

    set family times
    set weight medium
    set slant r
    set pixels 12

    if { $bold   } {set weight bold} 
    if { $italic } {set slant i} else {set slant r} 
    if { $head1  } {set pixels 24}
    if { $head2  } {set pixels 24}
    if { $head3  } {set pixels 18}
    if { $head4  } {set pixels 18}
    if { $head5  } {set pixels 14}
    if { $head6  } {set pixels 14}

    if { $preformat || $code } {
	set weight medium
	set slant r
	set family courier
	set pixels 14
    }
    set font "-*-$family-$weight-$slant-normal--$pixels-*-75-75-*-*-*-*"
    return $font
}

#
#  get_link_value:
#
#  purpose:  takes in a tag that should be in the form of an HTML link and 
#            returns the value of the reference.
#
#  note:     This is a very poor way of doing this, but it works in most 
#            instances.  If links are not linkt to the appropritae pages 
#            then this function might be the culprit.
#
proc get_link_value { input_string } {
    set data [string trim [string trim [lindex [split $input_string =] 1] >] \"]
    return $data
}

#
#  do_tag:
#
#  purpose:  Given a tag as an argument this procedure will deal with the tag 
#            in the appropriate manner.
#
#  note:     The switch statement at the heart of this is a bit cumbersome and
#            could probably have each of its cases made into seperate 
#            procedures.
#
proc do_tag {tag} {
    global current 
    global mark_start
    global mark_end
    global current_line
    global split_data
    global tab_stop
    global preformat
    global code
    global bold
    global italic
    global head1
    global head2
    global head3
    global head4
    global head5
    global head6
    global tag_number
    global bold_start
    global bold_end
    global italics_start
    global italics_end
    global head1_start
    global head1_end
    global head2_start
    global head2_end
    global head3_start
    global head3_end
    global head4_start
    global head4_end
    global head5_start
    global head5_end
    global head6_start
    global head6_end
    global center_start
    global center_end
    global link_start
    global link_end
    global link_value
    global under_start
    global under_end
    global over_start
    global over_end
    global current_directory
    global startup_directory
    set real_tag $tag
    set tag [string toupper $tag]
    #
    #  In this switch statement we will deal with each of the tags 
    #  There are quite a few tags that we don't recognize, but we 
    #  should still get pretty good results
    #
    switch -glob -- $tag {

	<HR*> {
	    # Horizontal Rule (Yes I know it is cheesy)
	    set temp [.help.text index insert]
	    .help.text insert insert "\n_____________________________________________________________________________________________________________________________________________________________________\n"
	    .help.text tag add my_tag$tag_number $temp [ .help.text index insert ]
	    .help.text tag configure my_tag$tag_number -wrap none
	    incr tag_number
	}
	<BR> {
	    # Line Break
	    .help.text insert insert "\n"
	}
	<H1*> {
	    # Level 1 Heading Start
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    set head1_start [.help.text index insert]
	    set head1 1
	}
	</H1> {
	    # Level 1 Heading End
	    set head1_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head1_start $head1_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head1 0
	    incr tag_number
	} 
	<H2*> {
	    # Level 2 Heading Start
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    set head2_start [.help.text index insert]
	    set head2 1
	    .help.text insert insert "\n"
	}
	</H2> {
	    # Level 2 Heading End
	    .help.text insert insert "\n"
	    set head2_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head2_start $head2_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head2 0
	    incr tag_number
	}
	<H3*> {
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    # Level 3 Heading Start
	    set head3_start [.help.text index insert]
	    set head3 1
	    .help.text insert insert "\n"
	}
	</H3> {
	    # Level 3 Heading End
	    .help.text insert insert "\n"
	    set head3_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head3_start $head3_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head3 0
	    incr tag_number
	}

	<H4*> {
	    # Level 4 Heading Start
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    set head4_start [.help.text index insert]
	    set head4 1
	    .help.text insert insert "\n"
	}
	</H4> {
	    # Level 4 Heading End
	    .help.text insert insert "\n"
	    set head4_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head4_start $head4_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head4 0
	    incr tag_number
	}
	
	<H5*> {
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    # Level 5 Heading Start
	    set head5_start [.help.text index insert]
	    set head5 1
	    .help.text insert insert "\n"
	}
	</H5> {
	    # Level 5 Heading End
	    .help.text insert insert "\n"
	    set head5_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head5_start $head5_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head5 0
	    incr tag_number
	}
	<H6*> {
	    # Level 6 Heading Start
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    set head6_start [.help.text index insert]
	    set head6 1
	    .help.text insert insert "\n"
	}
	</H6> {
	    # Level 6 Heading End
	    .help.text insert insert "\n"
	    set head6_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $head6_start $head6_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set head6 0
	    incr tag_number
	}
	<CODE> {
	    # Preformated text all we should do is display this in the window in a fixed width font
	    set code_start [.help.text index insert]
	    set code 1
	    set line ""
	    #	    .help.text insert insert "\n"
	    while {![string match [string toupper $line] "</CODE>"]} {
		.help.text insert insert $line
		incr current_line
		set line [lindex $split_data $current_line]
		#	.help.text insert insert "\n"
	    }
	    set code_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $code_start $code_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set code 0
	    incr tag_number
	}




	<PRE> {
	    # This isn't exactly displayed right
	    set preformat_start [.help.text index insert]
	    set preformat 1
	    set line ""
	    .help.text insert insert "\n"
	    while {![string match [string toupper $line] "</PRE>"]} {
		.help.text insert insert $line
		incr current_line
		set line [lindex $split_data $current_line]
		.help.text insert insert "\n"
	    }
	    set preformat_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $preformat_start $preformat_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set preformat 0
	    incr tag_number
	}
	</PRE> {}
	<P> {
	    # Paragraph
	    .help.text insert insert "\n\n"
	}
	</P> {}
	<STRONG> -
	<B> {
	    # Bold Start
	    set bold 1
	    set bold_start [.help.text index insert]
	}
	</STRONG> -
	</B> {
	    # Bold End
	    set bold_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $bold_start $bold_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set bold 0
	    incr tag_number
	}
	<VAR> -
	<EM> -
	<CITE> -
	<I> {
	    # Italics Start
	    set italic 1
	    set italics_start [.help.text index insert]
	}
	</VAR> -
	</EM> -
	</CITE> -
	</I> {
	    # Italics End
	    set italics_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $italics_start $italics_end
	    .help.text tag configure my_tag$tag_number -font [get_font]
	    set italics 0
	    incr tag_number
	}
	<U> {
	    # Underline Start
	    set under_start [.help.text index insert]
	    
	}
	</U> {
	    # Underline End
	    set under_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $under_start $under_end
	    .help.text tag configure my_tag$tag_number -underline 1
	    incr tag_number
	}
	<STRIKE> {
	    set over_start [.help.text index insert]
	}
	</STRIKE> {
	    set over_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $over_start $over_end
	    .help.text tag configure my_tag$tag_number -overstrike 1
	    incr tag_number
	}
	<OL*> -
	<DL*> -
	<UL*> { 
	    # Unordered List Start
	    incr tab_stop 
	}
	<OL> { 
	    # Ordered List Start
	    incr tab_stop 
	}
	</OL> -
	</DL> -
	</UL> { 
	    # Unordered List End
	    decr tab_stop 
	}
	
	<DT> -
	<LI> {
	    # List Item Start (ordered/unordered)
	    if { 0 != [ lindex [ split [ .help.text index insert ] . ] 1 ] } {
		.help.text insert insert "\n"
	    }
	    .help.text insert insert "    "
	}
	</LI> {
	    # List Item End (ordered/unordered)
	    .help.text insert insert "\n"
	}
	<DD> {
	    # Don't exactly know what this tag is, but it formats similar to 
	    #
	    # netscape if I do this
	    .help.text insert insert "\n"
	}
	</DD> {}
	"<A *>" {
	    # Link Start
	    #
	    # This code is quite shakey.  So if you see problems with links 
	    # not working it is probably due to this code ore something called 
	    # from here.  It might be a while before I get the name= portion of 
	    # this to work
	    set link_value [ get_link_value $real_tag ]
	    
	    #	    if {([string index $link_value 0] != "/") && ![is_absolute $link_value]} {}
	    if {![is_site_absolute $link_value] && ![is_absolute $link_value]} {
		set link_value $current_directory/$link_value
	    } elseif { [is_absolute $link_value] } {
		#		puts "Absolute link:  $link_value"
	    } elseif { [ is_target $link_value ] } {
		#		puts "Target:  $link_value"
	    } elseif { [is_site_absolute $link_value] } {
		set link_value $startup_directory$link_value
		if { [ is_directory $link_value] } {
		    set foo "index.html"
		    set link_value $link_value$foo
		}
	    }
	    
	    set link_start [.help.text index insert]
	}
	</A> {
	    # Link End
	    #
	    # See comments for previous tag
	    set link_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $link_start $link_end
	    ###################################
	    #    set history [concat $file_name $history]
	    ###################################
	    if { ( ![ is_absolute $link_value ] ) && ( ![ is_target $link_value ] ) } {
		.help.text tag configure my_tag$tag_number -underline 1 -foreground red
		.help.text tag bind my_tag$tag_number <Button-1> "do_link $link_value my_tag$tag_number" 
	    .help.text tag bind my_tag$tag_number <Enter> { .help.text configure -cursor { gumby black green } }
		.help.text tag bind my_tag$tag_number <Leave> { .help.text configure -cursor xterm }
	    } else {
		.help.text tag configure my_tag$tag_number -underline 1 -foreground navy
		.help.text tag bind my_tag$tag_number <Button-1> "exec netscape $link_value &"
		.help.text tag bind my_tag$tag_number <Enter> { .help.text configure -cursor { gumby black green } }
		.help.text tag bind my_tag$tag_number <Leave> { .help.text configure -cursor xterm }
	    }
	    incr tag_number
	}
	<FONT*> {}
	</FONT> {}
	<CENTER> {
	    # Center Text Start
	    set center_start [.help.text index insert]
	    .help.text insert insert "\n"}
	</CENTER> {
	    # Center Text End
	    .help.text insert insert "\n"
	    set center_end [.help.text index insert]
	    .help.text tag add my_tag$tag_number $center_start $center_end
	    .help.text tag configure my_tag$tag_number -justify center
	    incr tag_number
	}
	<TITLE> {
	    # Document Title
	    set title ""
	    set line ""
	    while { ![string match [string toupper $line] "</TITLE>"]} {
		set title "$title $line"
		incr current_line
		set line [lindex $split_data $current_line]
	    }
	    wm title .help $title
	}
	<META* {
	    set the_line $tag
	    while {![ string match {*>} $the_line] } {
		incr current_line
		set the_line [ lindex $split_data $current_line ]
	    }
	}
	"<BODY*>" {
	    # Body Tag
	    # 
	    # This might be modified to deal with foreground background coloring etc
	}
	"</BODY>" {}
	<BLOCKQUOTE> {}
	</BLOCKQUOTE> {}
	<MENU*> {}
	</MENU> {}
	<HTML> {}
	</HTML> {}
	<ADDRESS> {
	    .help.text insert insert "\n"
	}
	</ADDRESS> {}
	<HEAD> {}
	</HEAD> {}
	<!*> {}
	<TABLE*> {}
	</TABLE> {}
	<TR*> {}
	</TR> {}
	<TD*> {}
	</TD> {}
	</DT> {}
	<IMG*> {
	    # Image
	    set alt " |*image*| "
	    foreach i [ split [string trim [string trim $real_tag] <>] ] {
		if { [lindex [split $i =] 0] == "alt" } {
		    set alt [ string trim [ lindex [ split $i = ] 1 ] \" ] 
		}
	    }
	    .help.text insert insert " $alt "
	    
	}
	<FORM*> {}
	</FORM> {}
       	<OPTION*> {}
	<SELECT*> {}
	</SELECT> {}
	<INPUT*> {}
	default {
	    # Unknown Tags
	    puts "Unknown Tag:  $tag"
	}
    }

}

#
# do_link
#
# Purpose:  This function takes a link and a tag as parameters.  
#           It then updates the look of the link, updates the 
#           history, and displays the link in the window.
#
proc do_link { link tag } {
    global current_directory
    global history
    global current_history

    if { [ file isfile $link ] } { 
	set current_directory [ file dirname $link ]
	.help.text tag configure $tag -foreground orange
	update

	if { $current_history == 0 } {
	    set history [ concat $link $history ]
	} else {
	    set history [ concat $link [lrange $history $current_history end ] ]
	    set current_history 0
	}
	display_html $link
    } else { 
	puts "$link_value is not a valid link "
    }
}

#
# These are some utility functions used to determine the nature of a link
#
#
# Returns 1 if url is an absolute link
proc is_absolute { url } { return [string match {*:*} $url] }
# Returns 1 if url contains a target
proc is_target { url } { return [string match {*\#*} $url] }
# Returns 1 if url is site absolute
proc is_site_absolute { url } { return [string match {/*} $url] }
# Returns 1 if url is a directory
proc is_directory { url } { return [string match {*/} $url] }
# Returns 1 if line contains a tag
proc is_tag { line } { return [expr [ string match {*<*} $line ] || [ string match {*>*} $line ] ] }
# Returns 1 if line is a complete tag
proc is_good_tag { line } { return { [regexp {<(.*)>} [string trim $line] ] } }

#
#  Ensures that all tags start and stop on the same line
#
proc fix_tags {} {
    global split_data
    set list_size [llength $split_data ]
    set index 0
    while { $index < $list_size } {
	set line [lindex $split_data $index]
	if { [expr [expr ![is_tag $line]] ||  [ is_good_tag $line ]  ]} {
		lappend return_list $line
	} else {
	    while { [expr ![ is_good_tag $line ] ] } {
		incr index
		set line "$line [lindex $split_data $index]"
	    }
	    lappend return_list $line
	}
	incr index
    }
    set split_data $return_list
}


proc go_back  {} {
    global current_history
    global history
    if { [ expr [ llength $history ] - 1 ] > $current_history } {
	incr current_history
	display_html [ lindex $history $current_history ]	
    }
}



proc go_forward {} {
    global current_history
    global history
    if { ( [ llength $history ] > 1 ) && ( $current_history > 0 ) } {
	decr current_history
	display_html [ lindex $history $current_history ]	
    }
}

#
#  display_html:  
#
#  purpose:  this is the main body of the code it opens the file and parses 
#            through it
#
proc display_html { file_name } {
    global history
    global current_history
    global current_line
    global split_data
    global tab_stop
    global current_directory
    global normalBG


    if [winfo exists .help.text] {
	.help.text configure -state normal
	.help.text delete 1.0 end
    } else {
	toplevel .help
	wm withdraw .
	wm geometry .help +0+0
	text .help.text \
	    -yscrollcommand {.help.yscroll set} \
	    -wrap word -font [get_font] \
	    -fg black -bg $normalBG

#	.help.text configure -state disabled

#	scrollbar .help.xscroll -orient horizontal -command {.help.text xview}
	scrollbar .help.yscroll -orient vertical -command {.help.text yview}
#	text .help.text2 -xscrollcommand {.help.xscroll2 set} -yscrollcommand {.help.yscroll2 set} -wrap word
#	scrollbar .help.xscroll2 -orient horizontal -command {.help.text2 xview}
#	scrollbar .help.yscroll2 -orient vertical -command {.help.text2 yview}
	frame .help.buttons -relief ridge
	button .help.buttons.back -text "Back" -command { go_back }
	button .help.buttons.forward -text "Forward" -command { go_forward }
	button .help.buttons.exit -text "Exit" -command {destroy .help}
	pack .help.buttons.back .help.buttons.forward .help.buttons.exit -side left -fill x
	grid columnconfigure .help 0 -weight 1
	grid columnconfigure .help 1 -weight 0
	grid rowconfigure .help 0 -weight 1
	grid rowconfigure .help 1 -weight 0
#	grid rowconfigure .help 2 -weight 0
#	grid rowconfigure .help 3 -weight 1
#	grid rowconfigure .help 4 -weight 0
	grid .help.text -row 0 -column 0 -sticky nsew
#	grid .help.xscroll -row 1 -column 0 -sticky ew
	grid .help.yscroll -row 0 -column 1 -sticky ns
	grid .help.buttons -row 1 -column 0 -columnspan 2
#	grid .help.text2 -row 3 -column 0 -sticky nsew
#	grid .help.xscroll2 -row 4 -column 0 -sticky ew
#	grid .help.yscroll2 -row 3 -column 1 -sticky ns
#	bind .help <Key-q> {exit}
	set current_history 0
	set history $file_name

	bind .help.text <Up> {
	    break
	}

	bind .help.text <Down> {
	    break
	}

	bind .help.text <Left> {
	    break
	}

	bind .help.text <Right> {
	    break
	}

	bind .help.text <B1-Motion> {
	    break
	}

    }
    

    #
    #  We need to keep track of where the page is located.  This makes 
    #  relative links much easier.
    #
    set current_directory [ file dirname $file_name ]
    if {$current_directory == "."} {
	set current_directory [pwd]
    }
   
    set file_handle [open $file_name]
    # read the file into a variable
    set raw_data [read $file_handle]
    # ouput the raw file into a text widget so that we can compare the input to the output
    # this only shows up if we have gridded these widgets
#    .help.text2 delete 1.0 end
#    .help.text2 insert insert $raw_data
    #  Make sure that all tags are on a line of thier own.
    #  This should probably be done with regsub, but I was 
    #  having trouble getting that to work.
    set temp_data [join [split $raw_data "<"] "\n<"]
    set temp_data2 [join [split $temp_data ">"] ">\n"]
    #  Turn this into a list split on the newline
    set split_data [split $temp_data2 "\n"]
    
    set tab_stop 0
    set longtag 0
    set current_line 0
    set length_of_data [llength $split_data]
    

    fix_tags

    #  this loop should probably be moved to a seperate window in order to simplify this procedure
    while { $current_line < $length_of_data } {
	#  Remove leading and trailing white space.
	set i [string trim [ lindex $split_data $current_line ] ]
	#  Skip blank lines.
	if {$i == ""} {
	    incr current_line
	    continue
	}
    
	
	#  Check to see what kind of data we have, and then deal with it appropriately
	if { [regexp {^</(.*)} $i]} {
	    do_tag $i
	} elseif { [regexp {^<(.*)} $i] } {
	    do_tag $i
	} else {
	    .help.text insert insert " "
	    .help.text insert insert $i
	    #  Found standard text
	}
	# This update will slow things down, but since the 
	update 
	incr current_line
    }
    .help.text configure -state disabled
}


#
#  I used these to test the code
#
#display_html /usr/local/etc/httpd/htdocs/topics/tcltk/tutorial-noplugin/index.html
#display_html /net/hume/root/tmp/html_library-0.1/html/sample.html
#set file_handle [open /usr/local/etc/httpd/htdocs/topics/tcltk/index.html r]
#display_html /usr/local/etc/httpd/htdocs/index.html
#display_html /usr/local/etc/httpd/htdocs/about.html
#display_html /users/cdy/cdy/smart-gdb/gui/html/links.html
# display_html index.html
# Give a way to exit from script

