#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# Basic routines for the SmartGDB GUI
# Charles Dymond <cdy@ittc.ukans.edu>
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# default colors for highlighted program text, 
# highlighted breakpoints, program text, command window 
# text, command window prompts, and command window 
# insertion cursor
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-

# Highlight foreground 
global highlightFG 
set highlightFG black

# Highlight background 
global highlightBG
set highlightBG yellow

# Breakpoint foreground 
global breakFG
set breakFG white

# Breakpoint background
global breakBG
set breakBG red

# Standard text foreground
global normalFG
set normalFG blue

# Standard text background
global normalBG
set normalBG white

# Command window foreground
global commandFG
set commandFG black

# Command window background
global commandBG
set commandBG white

# Prompt foreground
global promptFG
set promptFG red

# Prompt background
global promptBG
set promptBG white

# Script prompt foreground
global insertFG
set insertFG blue

# Script prompt background
global insertBG
set insertBG white

# Command window cursor
global commandCursor
set commandCursor red


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# Colours a line of text in a given window 
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc colour_text { name line colour } {

    if { $line != 0 } { 
	$name configure -state normal
	$name mark set lfirst $line.0
	$name mark set llast $line.end
	$name tag add fl lfirst llast
	$name tag configure fl -foreground $colour
	$name configure -state disabled
    }
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# inserts text into a text widget, after first deleting
# any previously existing text in that widget
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc insert_text { windowName text} {

   $windowName configure -state normal
   $windowName delete 1.0 end
   $windowName mark set insert 1.0
   $windowName insert insert $text
   $windowName configure -state disabled
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# returns the line number from the output generated by
# the 'Step' & 'Next' commands
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc get_line_number { output } {

    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # get the line number
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    set first_ch [string index $output 0]
    set line_num ""
    set pos 1
    set tmp1 [ lindex [ split $output \n ] 0] 
    
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # line number is at start of 
    # line one of the output
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    if { ( $first_ch >= "0" )&&( $first_ch <= "9") } {
	set char $first_ch
       
	while { ($char >="0")&&($char <= "9")  } {
	    append line_num  $char 
	    set char [string index $tmp1 $pos]	    
	    incr pos
	}
    } else {
	set pos [string length $tmp1]
	set pos [expr $pos-1]
	set char [string index $tmp1 $pos]
	
	if { ($char >= "0" )&&( $char <= "9") } {
	    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
	    # number is at end of line one
	    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
	    
	    while { $char != ":" } {
		set line_num "$char$line_num"
		set pos [expr $pos-1]
		set char [string index $tmp1 $pos]	    
	    }
	} else {
	    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
	    # number is at start of line 3
	    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
	    set tmp2 [ lindex [ split $output \n ] 1]
	    set pos [string length $tmp2]
	    set pos [expr $pos-1]
	    set char [string index $tmp2 $pos]
	    
	    while { $char != ":" } {
		set line_num "$char$line_num"
		set pos [expr $pos-1]
		set char [string index $tmp2 $pos]       
	    }
	}
    }
    return $line_num
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# returns the contents of the line following the given
# line number
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc get_next_line { name line } {

    global lastLineSelected
    set lastLineSelected [incr lastLineSelected]
    set line [get_selected_line $name [incr line]]
    set lastLineSelected [expr $lastLineSelected-1]

    return $line
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# returns the contents of the entire line selected
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc get_selected_line { name line } {

    $name mark set lfirst $line.0
    $name mark set llast $line.end
    set line [$name get lfirst llast]
    return $line
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# Procedure: highlight line
# Purpose: this procedure highlights 
# an entire line of source 
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc highlight_line { name line_num } {

    global selectedLineNumber
    global lastWindowSelected
    global lastLineSelected
    global lastBline
    global lastMline
    global highlightFG
    global highlightBG
    global lastSline

    set lastLine ""

    if { $name ==".smartgdb.text_frame.text" } {
	set lastLine $lastMline
	set lastMline $line_num
    } elseif { $name == ".breaks.txt" } {
	set lastLine $lastBline
	set lastBline $line_num
    } elseif { $name == ".scripts.txt" } {
	set lastLine $lastSline
	set lastSline $line_num
    }  else { return } 
    
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # first unhighlight any previously
    # highlighted line
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-	
    if { $lastLine != 0 } {
	    $name mark set lfirst $lastLine.0
	    $name mark set llast $lastLine.end
	    $name tag remove ls lfirst llast
	}
    
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # now highlight the selected line
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    $name mark set lfirst $line_num.0
    $name mark set llast $line_num.end
    $name tag add ls lfirst llast
    $name tag configure ls -background $highlightBG \
	-foreground $highlightFG -relief raised \
	-borderwidth 1
 
    set lastLineSelected $line_num
    set selectedLineNumber $line_num
    set lastWindowSelected $name

    $name see $line_num.0
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-	
# returns line number of the current
# position of the pointer (if over a
# text widget)
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-	
proc get_pos { name } {

    global lastLineSelected

    $name mark set here current
    set lineinfo [ $name dump here]
    set tmp2 [ lindex [split $lineinfo " "] 2 ]
    set line_num [lindex [split $tmp2 .] 0]
    
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    # line_num is now the line number
    # of the line clicked on.
    #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    set lastLineSelected $line_num
    return $line_num
}


#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
# given a file name this procedure will display that
# file's listing in the main window
#+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
proc display_file { filename } {
    global lastLineSelected
    global currentSourceFile
    global lastMline
    set currentSourceFile $filename
    wm title .smartgdb "$currentSourceFile"  
    set foo "[ l $filename:1,10000 ]"

    insert_text .smartgdb.text_frame.text $foo

    set lastLineSelected 0
    set lastMline 0
}